@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,15 +17,13 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FtpDirList.h,v 1.12 2008/11/27 05:56:17 lav Exp $ */
-
 #ifndef FTPDIRLIST_H
 #define FTPDIRLIST_H
 
 class FtpDirList : public DirList
 {
    SMTaskRef<IOBuffer> ubuf;
-   xstring_ca pattern;
+   const xstring_ca pattern;
 
    bool TryEPLF(const char *line,int len);
    bool TryMLSD(const char *line,int len);
