@@ -111,7 +111,7 @@ final:
    if(stats.bytes)
       s.appendf("%s%s\n",tab,CopyJob::FormatBytesTimeRate(stats.bytes,transfer_time_elapsed));
    if(stats.del_dirs || stats.del_files || stats.del_symlinks)
-      s.appendf(plural(flags&DELETE ?
+      s.appendf(plural(FlagSet(DELETE) ?
 	       "%sRemoved: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n"
 	      :"%sTo be removed: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
 	      stats.del_dirs,stats.del_files,stats.del_symlinks),
@@ -326,9 +326,9 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 FileInfo *old=target_set->FindByName(FileCopy::TempFileName(file->name));
 	 if(old)
 	 {
-	    if((flags&CONTINUE)
+	    if(FlagSet(CONTINUE)
 	    && old->Has(file->TYPE) && old->filetype==old->NORMAL
-	    && (flags&IGNORE_TIME ||
+	    && (FlagSet(IGNORE_TIME) ||
 		(file->Has(file->DATE) && old->Has(old->DATE)
 		&& file->date + file->date.ts_prec < old->date - old->date.ts_prec))
 	    && file->Has(file->SIZE) && old->Has(old->SIZE)
@@ -350,7 +350,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    else
 	       stats.new_files++;
 	 }
-	 else if(flags&ONLY_EXISTING)
+	 else if(FlagSet(ONLY_EXISTING))
 	 {
 	    Report(_("Skipping file `%s' (only-existing)"),source_name_rel);
 	    goto skip;
@@ -434,7 +434,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    old=target_set->FindByName(file->name);
 	 if(!old)
 	 {
-	    if(flags&ONLY_EXISTING)
+	    if(FlagSet(ONLY_EXISTING))
 	    {
 	       Report(_("Skipping directory `%s' (only-existing)"),target_name_rel);
 	       goto skip;
@@ -446,7 +446,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 }
 	 if(target_is_local && !script_only)
 	 {
-	    if((flags&RETR_SYMLINKS?stat:lstat)(target_name,&st)!=-1)
+	    if((FlagSet(RETR_SYMLINKS)?stat:lstat)(target_name,&st)!=-1)
 	    {
 	       if(S_ISDIR(st.st_mode))
 	       {
@@ -493,7 +493,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
       }
       case(FileInfo::SYMLINK):
       {
-	 if(flags&NO_SYMLINKS)
+	 if(FlagSet(NO_SYMLINKS))
 	    goto skip;
 
 	 if(!file->symlink)
@@ -557,7 +557,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 }
 	 else
 	 {
-	    if(flags&ONLY_EXISTING)
+	    if(FlagSet(ONLY_EXISTING))
 	    {
 	       Report(_("Skipping symlink `%s' (only-existing)"),target_name_rel);
 	       goto skip;
@@ -578,31 +578,34 @@ skip:
    return;
 }
 
-void  MirrorJob::InitSets(Ref<FileSet>& source,const FileSet *dest)
+void  MirrorJob::InitSets()
 {
-   if(FlagSet(TARGET_FLAT) && !parent_mirror && dest)
-      source->Sort(FileSet::BYNAME_FLAT);
+   if(FlagSet(TARGET_FLAT) && !parent_mirror && target_set)
+      source_set->Sort(FileSet::BYNAME_FLAT);
 
-   source->Count(NULL,&stats.tot_files,&stats.tot_symlinks,&stats.tot_files);
+   source_set->Count(NULL,&stats.tot_files,&stats.tot_symlinks,&stats.tot_files);
 
-   to_rm=new FileSet(dest);
-   to_rm->SubtractAny(source);
+   to_rm=new FileSet(target_set);
+   to_rm->SubtractAny(source_set);
 
-   to_transfer=new FileSet(source);
+   if(FlagSet(DELETE_EXCLUDED) && target_set_excluded)
+      to_rm->Merge(target_set_excluded);
+
+   to_transfer=new FileSet(source_set);
 
    if(!FlagSet(TRANSFER_ALL)) {
-      same=new FileSet(source);
+      same=new FileSet(source_set);
 
       int ignore=0;
-      if(flags&ONLY_NEWER)
+      if(FlagSet(ONLY_NEWER))
 	 ignore|=FileInfo::IGNORE_SIZE_IF_OLDER|FileInfo::IGNORE_DATE_IF_OLDER;
       if(!FlagSet(UPLOAD_OLDER) && strcmp(target_session->GetProto(),"file"))
 	 ignore|=FileInfo::IGNORE_DATE_IF_OLDER;
-      if(flags&IGNORE_TIME)
+      if(FlagSet(IGNORE_TIME))
 	 ignore|=FileInfo::DATE;
-      if(flags&IGNORE_SIZE)
+      if(FlagSet(IGNORE_SIZE))
 	 ignore|=FileInfo::SIZE;
-      to_transfer->SubtractSame(dest,ignore);
+      to_transfer->SubtractSame(target_set,ignore);
 
       same->SubtractAny(to_transfer);
    }
@@ -617,7 +620,7 @@ void  MirrorJob::InitSets(Ref<FileSet>& source,const FileSet *dest)
    if(FlagSet(SCAN_ALL_FIRST)) {
       to_mkdir=new FileSet(to_transfer);
       to_mkdir->SubtractNotDirs();
-      to_mkdir->SubtractAny(dest);
+      to_mkdir->SubtractAny(target_set);
    }
 
    switch(recursion_mode) {
@@ -625,10 +628,10 @@ void  MirrorJob::InitSets(Ref<FileSet>& source,const FileSet *dest)
       to_transfer->SubtractDirs();
       break;
    case RECURSION_MISSING:
-      to_transfer->SubtractDirs(dest);
+      to_transfer->SubtractDirs(target_set);
       break;
    case RECURSION_NEWER:
-      to_transfer->SubtractNotOlderDirs(dest);
+      to_transfer->SubtractNotOlderDirs(target_set);
       break;
    case RECURSION_ALWAYS:
       break;
@@ -638,19 +641,19 @@ void  MirrorJob::InitSets(Ref<FileSet>& source,const FileSet *dest)
       to_transfer->ExcludeUnaccessible(source_session->GetUser());
 
    new_files_set=new FileSet(to_transfer);
-   new_files_set->SubtractAny(dest);
-   old_files_set=new FileSet(dest);
+   new_files_set->SubtractAny(target_set);
+   old_files_set=new FileSet(target_set);
    old_files_set->SubtractNotIn(to_transfer);
 
    to_rm_mismatched=new FileSet(old_files_set);
    to_rm_mismatched->SubtractSameType(to_transfer);
    to_rm_mismatched->SubtractNotDirs();
 
-   if(!(flags&DELETE))
+   if(!FlagSet(DELETE))
       to_transfer->SubtractAny(to_rm_mismatched);
 
-   if(FlagSet(TARGET_FLAT) && !parent_mirror && dest) {
-      source->Unsort();
+   if(FlagSet(TARGET_FLAT) && !parent_mirror && target_set) {
+      source_set->Unsort();
       to_transfer->UnsortFlat();
       to_transfer->SubtractDirs();
       same->UnsortFlat();
@@ -796,12 +799,12 @@ void MirrorJob::HandleListInfoCreation(const FileAccessRef& session,SMTaskRef<Li
    }
    list_info->UseCache(use_cache);
    int need=FileInfo::ALL_INFO;
-   if(flags&IGNORE_TIME)
+   if(FlagSet(IGNORE_TIME))
       need&=~FileInfo::DATE;
-   if(flags&IGNORE_SIZE)
+   if(FlagSet(IGNORE_SIZE))
       need&=~FileInfo::SIZE;
    list_info->Need(need);
-   if(flags&RETR_SYMLINKS)
+   if(FlagSet(RETR_SYMLINKS))
       list_info->FollowSymlinks();
 
    list_info->SetExclude(relative_dir,top_exclude?top_exclude:exclude);
@@ -867,7 +870,7 @@ int   MirrorJob::Do()
       if(target_is_local)
       {
 	 struct stat st;
-	 if((flags&RETR_SYMLINKS?stat:lstat)(target_dir,&st)!=-1)
+	 if((FlagSet(RETR_SYMLINKS)?stat:lstat)(target_dir,&st)!=-1)
 	 {
 	    if(S_ISDIR(st.st_mode))
 	    {
@@ -971,7 +974,7 @@ int   MirrorJob::Do()
       if(FlagSet(DEPTH_FIRST) && source_set && !target_set)
       {
 	 // transfer directories first
-	 InitSets(source_set,NULL);
+	 InitSets();
 	 to_transfer->Unsort();
 	 to_transfer->SubtractNotDirs();
 	 goto pre_WAITING_FOR_TRANSFER;
@@ -1015,7 +1018,7 @@ int   MirrorJob::Do()
 	 target_set->Merge(target_set_recursive);
 	 target_set_recursive=0;
       }
-      InitSets(source_set,target_set);
+      InitSets();
 
       to_transfer->CountBytes(&bytes_to_transfer);
       if(parent_mirror)
@@ -1113,7 +1116,7 @@ int   MirrorJob::Do()
       break;
 
    pre_TARGET_REMOVE_OLD:
-      if(flags&REMOVE_FIRST)
+      if(FlagSet(REMOVE_FIRST))
 	 goto pre_TARGET_CHMOD;
       set_state(TARGET_REMOVE_OLD);
       m=MOVED;
@@ -1136,7 +1139,7 @@ int   MirrorJob::Do()
 	    file=to_rm_mismatched->curr();
 	    to_rm_mismatched->next();
 	 }
-	 if(!file && (state==TARGET_REMOVE_OLD || (flags&REMOVE_FIRST)))
+	 if(!file && (state==TARGET_REMOVE_OLD || FlagSet(REMOVE_FIRST)))
 	 {
 	    file=to_rm->curr();
 	    to_rm->next();
@@ -1149,9 +1152,9 @@ int   MirrorJob::Do()
 	       goto pre_TARGET_CHMOD;
 	    goto pre_TARGET_MKDIR;
 	 }
-	 if(!(flags&DELETE))
+	 if(!FlagSet(DELETE))
 	 {
-	    if(flags&REPORT_NOT_DELETED)
+	    if(FlagSet(REPORT_NOT_DELETED))
 	    {
 	       const char *target_name_rel=dir_file(target_relative_dir,file->name);
 	       if(file->TypeIs(file->DIRECTORY))
@@ -1161,24 +1164,21 @@ int   MirrorJob::Do()
 	    }
 	    continue;
 	 }
+	 bool use_rmdir = (file->TypeIs(file->DIRECTORY)
+			   && recursion_mode==RECURSION_NEVER);
 	 if(script)
 	 {
-	    ArgV args("rm");
-	    if(file->TypeIs(file->DIRECTORY))
-	    {
-	       if(recursion_mode==RECURSION_NEVER)
-		  args.setarg(0,"rmdir");
-	       else
-		  args.Append("-r");
-	    }
+	    ArgV args(use_rmdir?"rmdir":"rm");
+	    if(file->TypeIs(file->DIRECTORY) && !use_rmdir)
+	       args.Append("-r");
 	    args.Append(target_session->GetFileURL(file->name));
 	    xstring_ca cmd(args.CombineQuoted());
 	    fprintf(script,"%s\n",cmd.get());
 	 }
 	 if(!script_only)
 	 {
-	    ArgV *args=new ArgV("rm");
-	    args->Append(file->name);
+	    ArgV *args=new ArgV(use_rmdir?"rmdir":"rm");
+	    args->Append(dir_file(".",file->name));
 	    args->seek(1);
 	    rmJob *j=new rmJob(target_session->Clone(),args);
 	    args->CombineTo(j->cmdline);
@@ -1186,10 +1186,7 @@ int   MirrorJob::Do()
 	    if(file->TypeIs(file->DIRECTORY))
 	    {
 	       if(recursion_mode==RECURSION_NEVER)
-	       {
-		  args->setarg(0,"rmdir");
 		  j->Rmdir();
-	       }
 	       else
 		  j->Recurse();
 	    }
@@ -1203,7 +1200,7 @@ int   MirrorJob::Do()
       break;
 
    pre_TARGET_CHMOD:
-      if(flags&NO_PERMS)
+      if(FlagSet(NO_PERMS))
 	 goto pre_FINISHING_FIX_LOCAL;
 
       to_transfer->rewind();
@@ -1255,7 +1252,7 @@ int   MirrorJob::Do()
 	 if(!script_only)
 	 {
 	    ArgV *a=new ArgV("chmod");
-	    a->Append(file->name);
+	    a->Append(dir_file(".",file->name));
 	    a->seek(1);
 	    ChmodJob *cj=new ChmodJob(target_session->Clone(),
 				 file->mode&~mode_mask,a);
@@ -1274,7 +1271,7 @@ int   MirrorJob::Do()
 	 const bool flat=FlagSet(TARGET_FLAT);
 	 to_transfer->Sort(FileSet::BYNAME_FLAT);
 	 to_transfer->LocalUtime(target_dir,/*only_dirs=*/true,flat);
-	 if(flags&ALLOW_CHOWN)
+	 if(FlagSet(ALLOW_CHOWN))
 	    to_transfer->LocalChown(target_dir,flat);
 	 if(!FlagSet(NO_PERMS) && same)
 	    same->LocalChmod(target_dir,get_mode_mask(),flat);
@@ -1377,7 +1374,7 @@ int   MirrorJob::Do()
 	 if(!script_only)
 	 {
 	    ArgV *args=new ArgV("rm");
-	    args->Append(file->name);
+	    args->Append(dir_file(".",file->name));
 	    args->seek(1);
 	    rmJob *j=new rmJob(source_session->Clone(),args);
 	    args->CombineTo(j->cmdline);
@@ -1401,7 +1398,7 @@ int   MirrorJob::Do()
       set_state(DONE);
       m=MOVED;
       bytes_transferred=0;
-      if(!parent_mirror && (flags&LOOP) && stats.HaveSomethingDone(flags) && !stats.error_count)
+      if(!parent_mirror && FlagSet(LOOP) && stats.HaveSomethingDone(flags) && !stats.error_count)
       {
 	 PrintStatus(0,"");
 	 printf(_("Retrying mirror...\n"));
@@ -1582,9 +1579,9 @@ void MirrorJob::SetOlderThan(const char *f)
 mode_t MirrorJob::get_mode_mask()
 {
    mode_t mode_mask=0;
-   if(!(flags&ALLOW_SUID))
+   if(!FlagSet(ALLOW_SUID))
       mode_mask|=S_ISUID|S_ISGID;
-   if(!(flags&NO_UMASK))
+   if(!FlagSet(NO_UMASK))
    {
       if(target_is_local)
       {
@@ -1640,7 +1637,7 @@ void MirrorJob::Statistics::Add(const Statistics &s)
    bytes       +=s.bytes;
    time	       +=s.time;
 }
-bool MirrorJob::Statistics::HaveSomethingDone(int flags)
+bool MirrorJob::Statistics::HaveSomethingDone(unsigned flags)
 {
    bool del=(flags&MirrorJob::DELETE);
    return new_files|mod_files|(del_files*del)|new_symlinks|mod_symlinks|(del_symlinks*del)|(del_dirs*del);
@@ -1795,6 +1792,7 @@ CMD(mirror)
       OPT_UPLOAD_OLDER,
       OPT_TRANSFER_ALL,
       OPT_TARGET_FLAT,
+      OPT_DELETE_EXCLUDED,
    };
    static const struct option mirror_opts[]=
    {
@@ -1856,11 +1854,12 @@ CMD(mirror)
       {"upload-older",no_argument,0,OPT_UPLOAD_OLDER},
       {"transfer-all",no_argument,0,OPT_TRANSFER_ALL},
       {"flat",no_argument,0,OPT_TARGET_FLAT},
+      {"delete-excluded",no_argument,0,OPT_DELETE_EXCLUDED},
       {0}
    };
 
    int opt;
-   int flags=0;
+   unsigned flags=0;
    int max_error_count=0;
 
    bool use_cache=false;
@@ -2106,6 +2105,9 @@ CMD(mirror)
       case(OPT_TARGET_FLAT):
 	 flags|=MirrorJob::TARGET_FLAT|MirrorJob::SCAN_ALL_FIRST|MirrorJob::DEPTH_FIRST;
 	 break;
+      case(OPT_DELETE_EXCLUDED):
+	 flags|=MirrorJob::DELETE_EXCLUDED;
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
       no_job:
