@@ -63,11 +63,13 @@ private:
 
    long long bytes_transferred;
    long long bytes_to_transfer;
+
    Ref<FileSet> target_set;
    Ref<FileSet> target_set_excluded;
    Ref<FileSet> source_set;
    Ref<FileSet> target_set_recursive;
    Ref<FileSet> source_set_recursive;
+
    Ref<FileSet> to_transfer;
    Ref<FileSet> to_mkdir;
    Ref<FileSet> same;
@@ -76,8 +78,8 @@ private:
    Ref<FileSet> old_files_set;
    Ref<FileSet> new_files_set;
    Ref<FileSet> to_rm_src;
-   void	 InitSets(Ref<FileSet>& src,const FileSet *dst);
-   bool only_dirs;
+   void InitSets(); // deduce above sets from source_set and target_set
+   bool only_dirs;  // to_transfer (or to_mkdir) contains directories only
 
    void RemoveSourceLater(const FileInfo *fi) {
       if(!remove_source_files)
@@ -123,7 +125,7 @@ private:
       Statistics();
       void Reset();
       void Add(const Statistics &);
-      bool HaveSomethingDone(int);
+      bool HaveSomethingDone(unsigned mirror_flags);
    };
    Statistics stats;
 
@@ -134,7 +136,7 @@ private:
     * and weight of a non-root mirror in global transfer_count otherwise. */
    int	 root_transfer_count;
 
-   int	 flags;
+   unsigned flags;
    recursion_mode_t recursion_mode;
    int	 max_error_count;
 
@@ -217,17 +219,18 @@ public:
       UPLOAD_OLDER=1<<21,
       TRANSFER_ALL=1<<22,
       TARGET_FLAT=1<<23,
+      DELETE_EXCLUDED=1<<24,
    };
-   void SetFlags(int f,bool v)
+   void SetFlags(unsigned f,bool v)
    {
       if(v)
 	 flags|=f;
       else
 	 flags&=~f;
    }
-   bool FlagsSet(int f)	   { return (flags&f)==f; }
-   bool FlagSet(int f)	   { return (flags&f); }
-   bool AnyFlagSet(int f)  { return (flags&f); }
+   bool FlagsSet(unsigned f)   const { return (flags&f)==f; }
+   bool FlagSet(unsigned f)    const { return (flags&f); }
+   bool AnyFlagSet(unsigned f) const { return (flags&f); }
 
    MirrorJob(MirrorJob *parent,FileAccess *f,FileAccess *target,
       const char *new_source_dir,const char *new_target_dir);
