@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: PtyShell.h,v 1.6 2008/11/27 05:56:25 lav Exp $ */
-
 #ifndef PTYSHELL_H
 #define PTYSHELL_H
 
@@ -28,7 +25,7 @@
 class PtyShell : public FDStream
 {
    Ref<ArgV> a;
-   Ref<ProcWait> w;
+   SMTaskRef<ProcWait> w;
    pid_t pg;
 
    xstring_c oldcwd;
