@@ -797,8 +797,7 @@ Job *CmdExec::builtin_lftp()
 	 cmd.append(';');
 	 break;
       case('c'):
-	 cmd.set_allocated(args->CombineCmd(args->getindex()-1));
-	 cmd.append("\n\n");
+	 args->CombineCmdTo(cmd,args->getindex()-1).append("\n\n");
 	 args->seek(args->count());
 	 break;
       case('d'):
@@ -2836,7 +2835,7 @@ CMD(ver)
       const char *symbol;
       enum type_t { STRING_PTR, FUNC0, INT8_8 } type;
       const char *skip_prefix;
-      typedef const char *(*func0)(int);
+      typedef const char *(*func0)(void *);
       const char *query() const
 	 {
 	    int v;
@@ -2850,7 +2849,7 @@ CMD(ver)
 	       str=*(const char**)sym_ptr;
 	       break;
 	    case FUNC0:
-	       str=((func0)sym_ptr)(0);
+	       str=((func0)sym_ptr)(NULL);
 	       break;
 	    case INT8_8:
 	       v=*(int*)sym_ptr;
@@ -2865,11 +2864,12 @@ CMD(ver)
    }
    static const libs[]=
    {
-      {"Readline",   "rl_library_version",   VersionInfo::STRING_PTR,0},
       {"Expat",	     "XML_ExpatVersion",     VersionInfo::FUNC0,     "expat_"},
-      {"OpenSSL",    "SSL_version_str",	     VersionInfo::STRING_PTR,"OpenSSL "},
       {"GnuTLS",     "gnutls_check_version", VersionInfo::FUNC0,     0},
+      {"idn2",	     "idn2_check_version",   VersionInfo::FUNC0,     0},
       {"libiconv",   "_libiconv_version",    VersionInfo::INT8_8,    0},
+      {"OpenSSL",    "SSL_version_str",	     VersionInfo::STRING_PTR,"OpenSSL "},
+      {"Readline",   "rl_library_version",   VersionInfo::STRING_PTR,0},
       {"zlib",	     "zlibVersion",	     VersionInfo::FUNC0,     0},
       {0}
    };
@@ -2884,11 +2884,10 @@ CMD(ver)
       snprintf(buf,sizeof(buf),", %s %s",scan->lib_name,v);
       int skip=need_comma?0:2;
       int w=mbswidth(buf+skip,mbflags);
-      if(col+w>=width)
+      if(col+w>=width && need_comma)
       {
 	 buf[1]='\n';
-	 col=w-2+skip;
-	 skip/=2;
+	 col=w-2;
       }
       else
 	 col+=w;
@@ -3039,7 +3038,7 @@ CMD(bookmark)
 CMD(echo)
 {
    xstring s;
-   s.set_allocated(args->Combine(1));
+   args->CombineTo(s,1);
    if(args->count()>1 && !strcmp(args->getarg(1),"-n"))
    {
       if(s.length()<=3)
@@ -3607,7 +3606,7 @@ CMD(eval)
    int base=optind;
    xstring cmd;
    if(!fmt)
-      cmd.set_allocated(args->Combine(optind));
+      args->CombineTo(cmd,optind);
    else
    {
       while(*fmt)
