@@ -52,7 +52,8 @@ static const char *FtpProxyValidate(xstring_c *p)
    if(url.user && !url.pass)
    {
       url.pass.set(GetPass(_("ftp:proxy password: ")));
-      p->set_allocated(url.Combine());
+      p->truncate();
+      url.CombineTo(*p);
    }
    return 0;
 }
@@ -218,7 +219,7 @@ static ResType lftp_vars[] = {
    {"ftp:passive-mode",		 "on",    ResMgr::BoolValidate,0},
    {"ftp:port-range",		 "full",  ResMgr::RangeValidate,0},
    {"ftp:port-ipv4",		 "",	  ResMgr::IPv4AddrValidate,0},
-   {"ftp:prefer-epsv",		 "yes",	  ResMgr::BoolValidate,0},
+   {"ftp:prefer-epsv",		 "no",	  ResMgr::BoolValidate,0},
    {"ftp:proxy",		 "",	  FtpProxyValidate,0},
    {"ftp:proxy-auth-type",	 "user",  FtpProxyAuthTypeValidate,0},
    {"ftp:rest-list",		 "no",	  ResMgr::BoolValidate,0},
@@ -314,8 +315,8 @@ static ResType lftp_vars[] = {
    {"net:idle",			 "3m",	  ResMgr::TimeIntervalValidate,0},
    {"net:limit-max",		 "0",	  ResMgr::UNumberValidate,0},
    {"net:limit-rate",		 "0:0",   ResMgr::UNumberPairValidate,0},
-   {"net:limit-total-max",	 "0",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
-   {"net:limit-total-rate",	 "0:0",   ResMgr::UNumberPairValidate,ResMgr::NoClosure},
+   {"net:limit-total-max",	 "0",	  ResMgr::UNumberValidate,0},
+   {"net:limit-total-rate",	 "0:0",   ResMgr::UNumberPairValidate,0},
    {"net:max-retries",		 "1000",  ResMgr::UNumberValidate,0},
    {"net:persist-retries",	 "0",	  ResMgr::UNumberValidate,0},
    {"net:no-proxy",		 "",	  0,ResMgr::NoClosure},
@@ -336,10 +337,10 @@ static ResType lftp_vars[] = {
    {"mirror:sort-by",		 "name",  SortByValidate,ResMgr::NoClosure},
    {"mirror:order",		 "*.sfv *.sig *.md5* *.sum * */", 0,ResMgr::NoClosure},
    {"mirror:parallel-directories", "yes", ResMgr::BoolValidate,ResMgr::NoClosure},
-   {"mirror:parallel-transfer-count", "1",ResMgr::UNumberValidate,ResMgr::NoClosure},
+   {"mirror:parallel-transfer-count", "0",ResMgr::UNumberValidate,0},
    {"mirror:exclude-regex",	 "(^|/)(\\.in\\.|\\.nfs)",ResMgr::ERegExpValidate,ResMgr::NoClosure},
    {"mirror:include-regex",	 "",	  ResMgr::ERegExpValidate,ResMgr::NoClosure},
-   {"mirror:use-pget-n",	 "1",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
+   {"mirror:use-pget-n",	 "0",	  ResMgr::UNumberValidate,0},
    {"mirror:set-permissions",	 "yes",   ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:dereference",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:skip-noaccess",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
